@@ -1,25 +1,16 @@
 /*
- * "$Id: template.c 5549 2006-05-19 19:39:28Z mike $"
+ * "$Id: template.c 7220 2008-01-14 22:12:58Z mike $"
  *
  *   CGI template function.
  *
+ *   Copyright 2007-2008 by Apple Inc.
  *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  * Contents:
  *
@@ -59,9 +50,16 @@ cgiCopyTemplateFile(FILE       *out,	/* I - Output file */
   FILE	*in;				/* Input file */
 
 
-  fprintf(stderr, "DEBUG: cgiCopyTemplateFile(out=%p, tmpl=\"%s\")\n", out,
+  fprintf(stderr, "DEBUG2: cgiCopyTemplateFile(out=%p, tmpl=\"%s\")\n", out,
           tmpl ? tmpl : "(null)");
 
+ /*
+  * Range check input...
+  */
+
+  if (!tmpl || !out)
+    return;
+
  /*
   * Open the template file...
   */
@@ -94,37 +92,34 @@ cgiCopyTemplateFile(FILE       *out,	/* I - Output file */
 void
 cgiCopyTemplateLang(const char *tmpl)	/* I - Base filename */
 {
-  int		i;			/* Looping var */
   char		filename[1024],		/* Filename */
-		locale[16];		/* Locale name */
+		locale[16],		/* Locale name */
+		*locptr;		/* Pointer into locale name */
   const char	*directory,		/* Directory for templates */
 		*lang;			/* Language */
   FILE		*in;			/* Input file */
 
 
-  fprintf(stderr, "DEBUG: cgiCopyTemplateLang(tmpl=\"%s\")\n",
+  fprintf(stderr, "DEBUG2: cgiCopyTemplateLang(tmpl=\"%s\")\n",
           tmpl ? tmpl : "(null)");
 
  /*
   * Convert the language to a locale name...
   */
 
+  locale[0] = '\0';
+
   if ((lang = getenv("LANG")) != NULL)
   {
-    for (i = 0; lang[i] && i < 15; i ++)
-      if (isalnum(lang[i] & 255) || lang[i] == '_')
-        locale[i] = tolower(lang[i]);
-      else if (lang[i] == '-')
-        locale[i] = '_';
-      else
-        break;
+    locale[0] = '/';
+    strlcpy(locale + 1, lang, sizeof(locale) - 1);
 
-    locale[i] = '\0';
+    if ((locptr = strchr(locale, '.')) != NULL)
+      *locptr = '\0';			/* Strip charset */
   }
-  else
-    locale[0] = '\0';
 
-  fprintf(stderr, "DEBUG: locale=\"%s\"...\n", locale);
+  fprintf(stderr, "DEBUG: lang=\"%s\", locale=\"%s\"...\n",
+          lang ? lang : "(null)", locale);
 
  /*
   * See if we have a template file for this language...
@@ -132,23 +127,26 @@ cgiCopyTemplateLang(const char *tmpl)	/* I - Base filename */
 
   directory = cgiGetTemplateDir();
 
-  snprintf(filename, sizeof(filename), "%s/%s/%s", directory, locale, tmpl);
-  if (access(filename, 0))
+  snprintf(filename, sizeof(filename), "%s%s/%s", directory, locale, tmpl);
+  if ((in = fopen(filename, "r")) == NULL)
   {
-    locale[2] = '\0';
+    locale[3] = '\0';
 
-    snprintf(filename, sizeof(filename), "%s/%s/%s", directory, locale, tmpl);
-    if (access(filename, 0))
+    snprintf(filename, sizeof(filename), "%s%s/%s", directory, locale, tmpl);
+    if ((in = fopen(filename, "r")) == NULL)
+    {
       snprintf(filename, sizeof(filename), "%s/%s", directory, tmpl);
+      in = fopen(filename, "r");
+    }
   }
 
-  fprintf(stderr, "DEBUG: Template file is \"%s\"...\n", filename);
+  fprintf(stderr, "DEBUG2: Template file is \"%s\"...\n", filename);
 
  /*
   * Open the template file...
   */
 
-  if ((in = fopen(filename, "r")) == NULL)
+  if (!in)
   {
     fprintf(stderr, "ERROR: Unable to open template file \"%s\" - %s\n",
             filename, strerror(errno));
@@ -240,7 +238,7 @@ cgi_copy(FILE *out,			/* I - Output file */
   int		uriencode;		/* Encode as URI */
 
 
-  fprintf(stderr, "DEBUG: %*sStarting at file position %ld...\n", indent, "",
+  fprintf(stderr, "DEBUG2: %*sStarting at file position %ld...\n", indent, "",
           ftell(in));
 
  /*
@@ -272,7 +270,7 @@ cgi_copy(FILE *out,			/* I - Output file */
 
       if (s == name && isspace(ch & 255))
       {
-        fprintf(stderr, "DEBUG: %*sLone { at %ld...\n", indent, "", ftell(in));
+        fprintf(stderr, "DEBUG2: %*sLone { at %ld...\n", indent, "", ftell(in));
 
         if (out)
 	{
@@ -284,7 +282,7 @@ cgi_copy(FILE *out,			/* I - Output file */
       }
 
       if (ch == '}')
-	fprintf(stderr, "DEBUG: %*s\"{%s}\" at %ld...\n", indent, "", name,
+	fprintf(stderr, "DEBUG2: %*s\"{%s}\" at %ld...\n", indent, "", name,
         	ftell(in));
 
      /*
@@ -348,7 +346,7 @@ cgi_copy(FILE *out,			/* I - Output file */
 
 	pos = ftell(in);
 
-        fprintf(stderr, "DEBUG: %*sLooping on \"%s\" at %ld, count=%d...\n",
+        fprintf(stderr, "DEBUG2: %*sLooping on \"%s\" at %ld, count=%d...\n",
 	        indent, "", name + 1, pos, count);
 
         if (count > 0)
@@ -364,7 +362,7 @@ cgi_copy(FILE *out,			/* I - Output file */
 	else
 	  cgi_copy(NULL, in, 0, '}', indent + 2);
 
-        fprintf(stderr, "DEBUG: %*sFinished looping on \"%s\"...\n", indent,
+        fprintf(stderr, "DEBUG2: %*sFinished looping on \"%s\"...\n", indent,
 	        "", name + 1);
 
         continue;
@@ -409,6 +407,8 @@ cgi_copy(FILE *out,			/* I - Output file */
 	{
 	  if (uriencode)
 	    cgi_puturi(outptr, out);
+	  else if (!strcasecmp(name, "?cupsdconf_default"))
+	    fputs(outptr, stdout);
 	  else
 	    cgi_puts(outptr, out);
         }
@@ -500,7 +500,7 @@ cgi_copy(FILE *out,			/* I - Output file */
         if (ch != '?')
 	{
 	  fprintf(stderr,
-	          "DEBUG: %*sBad terminator '%c' at file position %ld...\n",
+	          "DEBUG2: %*sBad terminator '%c' at file position %ld...\n",
 	          indent, "", ch, ftell(in));
 	  return;
 	}
@@ -530,7 +530,7 @@ cgi_copy(FILE *out,			/* I - Output file */
       }
 
       fprintf(stderr,
-              "DEBUG: %*sStarting \"{%s%c%s\" at %ld, result=%d...\n",
+              "DEBUG2: %*sStarting \"{%s%c%s\" at %ld, result=%d...\n",
 	      indent, "", name, op, compare, ftell(in), result);
 
       if (result)
@@ -539,10 +539,10 @@ cgi_copy(FILE *out,			/* I - Output file */
 	* Comparison true; output first part and ignore second...
 	*/
 
-        fprintf(stderr, "DEBUG: %*sOutput first part...\n", indent, "");
+        fprintf(stderr, "DEBUG2: %*sOutput first part...\n", indent, "");
 	cgi_copy(out, in, element, ':', indent + 2);
 
-        fprintf(stderr, "DEBUG: %*sSkip second part...\n", indent, "");
+        fprintf(stderr, "DEBUG2: %*sSkip second part...\n", indent, "");
 	cgi_copy(NULL, in, element, '}', indent + 2);
       }
       else
@@ -551,14 +551,14 @@ cgi_copy(FILE *out,			/* I - Output file */
 	* Comparison false; ignore first part and output second...
 	*/
 
-        fprintf(stderr, "DEBUG: %*sSkip first part...\n", indent, "");
+        fprintf(stderr, "DEBUG2: %*sSkip first part...\n", indent, "");
 	cgi_copy(NULL, in, element, ':', indent + 2);
 
-        fprintf(stderr, "DEBUG: %*sOutput second part...\n", indent, "");
+        fprintf(stderr, "DEBUG2: %*sOutput second part...\n", indent, "");
 	cgi_copy(out, in, element, '}', indent + 2);
       }
 
-      fprintf(stderr, "DEBUG: %*sFinished \"{%s%c%s\", out=%p...\n", indent, "",
+      fprintf(stderr, "DEBUG2: %*sFinished \"{%s%c%s\", out=%p...\n", indent, "",
               name, op, compare, out);
     }
     else if (ch == '\\')	/* Quoted char */
@@ -572,11 +572,11 @@ cgi_copy(FILE *out,			/* I - Output file */
       putc(ch, out);
 
   if (ch == EOF)
-    fprintf(stderr, "DEBUG: %*sReturning at file position %ld on EOF...\n",
+    fprintf(stderr, "DEBUG2: %*sReturning at file position %ld on EOF...\n",
 	    indent, "", ftell(in));
   else
     fprintf(stderr,
-            "DEBUG: %*sReturning at file position %ld on character '%c'...\n",
+            "DEBUG2: %*sReturning at file position %ld on character '%c'...\n",
 	    indent, "", ftell(in), ch);
 
   if (ch == EOF && term)
@@ -670,5 +670,5 @@ cgi_puturi(const char *s,		/* I - String to output */
 
 
 /*
- * End of "$Id: template.c 5549 2006-05-19 19:39:28Z mike $".
+ * End of "$Id: template.c 7220 2008-01-14 22:12:58Z mike $".
  */
