@@ -1,25 +1,16 @@
 /*
- * "$Id: cups.h 6550 2007-06-04 15:38:28Z mike $"
+ * "$Id: cups.h 7139 2007-12-19 01:43:31Z mike $"
  *
  *   API definitions for the Common UNIX Printing System (CUPS).
  *
+ *   Copyright 2007 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  *   This file is subject to the Apple OS-Developed Software exception.
  */
@@ -68,10 +59,10 @@ extern "C" {
  * Constants...
  */
 
-#  define CUPS_VERSION		1.0212
+#  define CUPS_VERSION		1.0306
 #  define CUPS_VERSION_MAJOR	1
-#  define CUPS_VERSION_MINOR	2
-#  define CUPS_VERSION_PATCH	12
+#  define CUPS_VERSION_MINOR	3
+#  define CUPS_VERSION_PATCH	6
 #  define CUPS_DATE_ANY		-1
 
 
@@ -107,7 +98,8 @@ enum cups_ptype_e			/* Not a typedef'd enum so we can OR */
   CUPS_PRINTER_NOT_SHARED = 0x200000,	/* Printer is not shared @since CUPS 1.2@ */
   CUPS_PRINTER_AUTHENTICATED = 0x400000,/* Printer requires authentication @since CUPS 1.2@ */
   CUPS_PRINTER_COMMANDS = 0x800000,	/* Printer supports maintenance commands @since CUPS 1.2@ */
-  CUPS_PRINTER_OPTIONS = 0xe6fffc	/* ~(CLASS | REMOTE | IMPLICIT) */
+  CUPS_PRINTER_DISCOVERED = 0x1000000,	/* Printer was automatically discovered and added @since CUPS 1.3@ */
+  CUPS_PRINTER_OPTIONS = 0x6fffc	/* ~(CLASS | REMOTE | IMPLICIT | DEFAULT | FAX | REJECTING | DELETE | NOT_SHARED | AUTHENTICATED | COMMANDS | DISCOVERED) */
 };
 
 typedef const char *(*cups_password_cb_t)(const char *);
@@ -243,6 +235,19 @@ extern int		cupsRemoveOption(const char *name, int num_options,
 			                 cups_option_t **options);
 extern cups_file_t	*cupsTempFile2(char *filename, int len);
 
+/**** New in CUPS 1.3 ****/
+extern ipp_t		*cupsDoIORequest(http_t *http, ipp_t *request,
+			                 const char *resource, int infile,
+					 int outfile);
+extern char		*cupsGetServerPPD(http_t *http, const char *name);
+extern int		cupsRemoveDest(const char *name,
+			               const char *instance,
+				       int num_dests, cups_dest_t **dests);
+extern void		cupsSetDefaultDest(const char *name,
+			                   const char *instance,
+					   int num_dests,
+					   cups_dest_t *dests);
+
 
 #  ifdef __cplusplus
 }
@@ -251,5 +256,5 @@ extern cups_file_t	*cupsTempFile2(char *filename, int len);
 #endif /* !_CUPS_CUPS_H_ */
 
 /*
- * End of "$Id: cups.h 6550 2007-06-04 15:38:28Z mike $".
+ * End of "$Id: cups.h 7139 2007-12-19 01:43:31Z mike $".
  */
