@@ -1,25 +1,19 @@
 /*
- * "$Id: auth.c 5948 2006-09-12 13:58:39Z mike $"
+ * "$Id: auth.c 7319 2008-02-15 23:26:51Z mike $"
  *
  *   Authorization routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2006 by Easy Software Products, all rights reserved.
+ *   Copyright 2007-2008 by Apple Inc.
+ *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
+ *
+ *   This file contains Kerberos support code, copyright 2006 by
+ *   Jelmer Vernooij.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  * Contents:
  *
@@ -49,9 +43,11 @@
  *   compare_locations()       - Compare two locations.
  *   cups_crypt()              - Encrypt the password using the DES or MD5
  *                               algorithms, as needed.
+ *   get_gss_creds()           - Obtain GSS credentials.
  *   get_md5_password()        - Get an MD5 password.
  *   pam_func()                - PAM conversation function.
  *   to64()                    - Base64-encode an integer value...
+ *   check_authref()           - Check an authorization services reference.
  */
 
 /*
@@ -79,6 +75,25 @@
 #ifdef HAVE_MEMBERSHIP_H
 #  include <membership.h>
 #endif /* HAVE_MEMBERSHIP_H */
+#ifdef HAVE_AUTHORIZATION_H
+#  include <Security/AuthorizationTags.h>
+#  ifdef HAVE_SECBASEPRIV_H
+#    include <Security/SecBasePriv.h>
+#  else
+extern const char *cssmErrorString(int error);
+#  endif /* HAVE_SECBASEPRIV_H */
+#endif /* HAVE_AUTHORIZATION_H */
+#ifdef HAVE_SYS_PARAM_H
+#  include <sys/param.h>
+#endif /* HAVE_SYS_PARAM_H */
+#ifdef HAVE_SYS_UCRED_H
+#  include <sys/ucred.h>
+typedef struct xucred cupsd_ucred_t;
+#  define CUPSD_UCRED_UID(c) (c).cr_uid
+#else
+typedef struct ucred cupsd_ucred_t;
+#  define CUPSD_UCRED_UID(c) (c).uid
+#endif /* HAVE_SYS_UCRED_H */
 
 
 /*
@@ -87,17 +102,23 @@
 
 static cupsd_authmask_t	*add_allow(cupsd_location_t *loc);
 static cupsd_authmask_t	*add_deny(cupsd_location_t *loc);
+#ifdef HAVE_AUTHORIZATION_H
+static int		check_authref(cupsd_client_t *con, const char *right);
+#endif /* HAVE_AUTHORIZATION_H */
 static int		compare_locations(cupsd_location_t *a,
 			                  cupsd_location_t *b);
 #if !HAVE_LIBPAM && !defined(HAVE_USERSEC_H)
 static char		*cups_crypt(const char *pw, const char *salt);
 #endif /* !HAVE_LIBPAM && !HAVE_USERSEC_H */
+#ifdef HAVE_GSSAPI
+static gss_cred_id_t	get_gss_creds(const char *service_name);
+#endif /* HAVE_GSSAPI */
 static char		*get_md5_password(const char *username,
 			                  const char *group, char passwd[33]);
 #if HAVE_LIBPAM
 static int		pam_func(int, const struct pam_message **,
 			         struct pam_response **, void *);
-#else
+#elif !defined(HAVE_USERSEC_H)
 static void		to64(char *s, unsigned long v, int n);
 #endif /* HAVE_LIBPAM */
 
@@ -155,8 +176,13 @@ cupsdAddLocation(const char *location)	/* I - Location path */
   * Initialize the record and copy the name over...
   */
 
-  temp->location = strdup(location);
-  temp->length   = strlen(temp->location);
+  if ((temp->location = strdup(location)) == NULL)
+  {
+    free(temp);
+    return (NULL);
+  }
+
+  temp->length = strlen(temp->location);
 
   cupsArrayAdd(Locations, temp);
 
@@ -236,7 +262,7 @@ cupsdAllowHost(cupsd_location_t *loc,	/* I - Location to add to */
     * Allow *interface*...
     */
 
-    temp->type             = AUTH_INTERFACE;
+    temp->type             = CUPSD_AUTH_INTERFACE;
     temp->mask.name.name   = strdup("*");
     temp->mask.name.length = 1;
   }
@@ -256,7 +282,7 @@ cupsdAllowHost(cupsd_location_t *loc,	/* I - Location to add to */
       *ifptr = '\0';
     }
 
-    temp->type             = AUTH_INTERFACE;
+    temp->type             = CUPSD_AUTH_INTERFACE;
     temp->mask.name.name   = strdup(ifname);
     temp->mask.name.length = ifptr - ifname;
   }
@@ -266,7 +292,7 @@ cupsdAllowHost(cupsd_location_t *loc,	/* I - Location to add to */
     * Allow name...
     */
 
-    temp->type             = AUTH_NAME;
+    temp->type             = CUPSD_AUTH_NAME;
     temp->mask.name.name   = strdup(name);
     temp->mask.name.length = strlen(name);
   }
@@ -279,9 +305,10 @@ cupsdAllowHost(cupsd_location_t *loc,	/* I - Location to add to */
  */
 
 void
-cupsdAllowIP(cupsd_location_t *loc,	/* I - Location to add to */
-             unsigned   address[4],	/* I - IP address to add */
-             unsigned   netmask[4])	/* I - Netmask of address */
+cupsdAllowIP(
+    cupsd_location_t *loc,		/* I - Location to add to */
+    const unsigned   address[4],	/* I - IP address to add */
+    const unsigned   netmask[4])	/* I - Netmask of address */
 {
   cupsd_authmask_t	*temp;		/* New host/domain mask */
 
@@ -295,7 +322,7 @@ cupsdAllowIP(cupsd_location_t *loc,	/* I - Location to add to */
   if ((temp = add_allow(loc)) == NULL)
     return;
 
-  temp->type = AUTH_IP;
+  temp->type = CUPSD_AUTH_IP;
   memcpy(temp->mask.ip.address, address, sizeof(temp->mask.ip.address));
   memcpy(temp->mask.ip.netmask, netmask, sizeof(temp->mask.ip.netmask));
 }
@@ -309,9 +336,9 @@ void
 cupsdAuthorize(cupsd_client_t *con)	/* I - Client connection */
 {
   int		type;			/* Authentication type */
-  char		*authorization,		/* Pointer into Authorization string */
-		*ptr,			/* Pointer into string */
-		username[65],		/* Username string */
+  const char	*authorization;		/* Pointer into Authorization string */
+  char		*ptr,			/* Pointer into string */
+		username[256],		/* Username string */
 		password[33];		/* Password string */
   const char	*localuser;		/* Certificate username */
   char		nonce[HTTP_MAX_VALUE],	/* Nonce value from client */
@@ -342,13 +369,19 @@ cupsdAuthorize(cupsd_client_t *con)	/* I - Client connection */
   */
 
   con->best = cupsdFindBest(con->uri, con->http.state);
+  con->type = CUPSD_AUTH_NONE;
 
   cupsdLogMessage(CUPSD_LOG_DEBUG2,
                   "cupsdAuthorize: con->uri=\"%s\", con->best=%p(%s)",
                   con->uri, con->best, con->best ? con->best->location : "");
 
-  if (con->best && con->best->type != AUTH_NONE)
-    type = con->best->type;
+  if (con->best && con->best->type != CUPSD_AUTH_NONE)
+  {
+    if (con->best->type == CUPSD_AUTH_DEFAULT)
+      type = DefaultAuthType;
+    else
+      type = con->best->type;
+  }
   else
     type = DefaultAuthType;
 
@@ -356,7 +389,7 @@ cupsdAuthorize(cupsd_client_t *con)	/* I - Client connection */
   * Decode the Authorization string...
   */
 
-  authorization = con->http.fields[HTTP_FIELD_AUTHORIZATION];
+  authorization = httpGetField(&con->http, HTTP_FIELD_AUTHORIZATION);
 
   cupsdLogMessage(CUPSD_LOG_DEBUG2, "cupsdAuthorize: Authorization=\"%s\"",
                   authorization);
@@ -364,17 +397,15 @@ cupsdAuthorize(cupsd_client_t *con)	/* I - Client connection */
   username[0] = '\0';
   password[0] = '\0';
 
-  if (type == AUTH_NONE)
+#ifdef HAVE_AUTHORIZATION_H
+  if (con->authref)
   {
-   /*
-    * No authorization required, return early...
-    */
-
-    cupsdLogMessage(CUPSD_LOG_DEBUG,
-                    "cupsdAuthorize: No authentication required.");
-    return;
+    AuthorizationFree(con->authref, kAuthorizationFlagDefaults);
+    con->authref = NULL;
   }
-  else if (!*authorization)
+#endif /* HAVE_AUTHORIZATION_H */
+
+  if (!*authorization)
   {
    /*
     * No authorization data provided, return early...
@@ -384,6 +415,120 @@ cupsdAuthorize(cupsd_client_t *con)	/* I - Client connection */
                     "cupsdAuthorize: No authentication data provided.");
     return;
   }
+#ifdef HAVE_AUTHORIZATION_H
+  else if (!strncmp(authorization, "AuthRef", 6) && 
+           !strcasecmp(con->http.hostname, "localhost"))
+  {
+    OSStatus		status;		/* Status */
+    int			authlen;	/* Auth string length */
+    AuthorizationItemSet *authinfo;	/* Authorization item set */
+
+   /*
+    * Get the Authorization Services data...
+    */
+
+    authorization += 7;
+    while (isspace(*authorization & 255))
+      authorization ++;
+
+    authlen = sizeof(nonce);
+    httpDecode64_2(nonce, &authlen, authorization);
+
+    if (authlen != kAuthorizationExternalFormLength)
+    {
+      cupsdLogMessage(CUPSD_LOG_ERROR,
+	              "External Authorization reference size is incorrect!");
+      return;
+    }
+
+    if ((status = AuthorizationCreateFromExternalForm(
+		      (AuthorizationExternalForm *)nonce, &con->authref)) != 0)
+    {
+      cupsdLogMessage(CUPSD_LOG_ERROR,
+		      "AuthorizationCreateFromExternalForm returned %d (%s)",
+		      (int)status, cssmErrorString(status));
+      return;
+    }
+
+    if ((status = AuthorizationCopyInfo(con->authref, 
+					kAuthorizationEnvironmentUsername, 
+					&authinfo)) != 0)
+    {
+      cupsdLogMessage(CUPSD_LOG_ERROR,
+		      "AuthorizationCopyInfo returned %d (%s)",
+		      (int)status, cssmErrorString(status));
+      return;
+    }
+  
+    if (authinfo->count == 1)
+      strlcpy(username, authinfo->items[0].value, sizeof(username));
+
+    cupsdLogMessage(CUPSD_LOG_DEBUG,
+                    "cupsdAuthorize: Authorized as %s using AuthRef",
+		    username);
+
+    AuthorizationFreeItemSet(authinfo);
+
+    con->type = CUPSD_AUTH_BASIC;
+  }
+#endif /* HAVE_AUTHORIZATION_H */
+#if defined(SO_PEERCRED) && defined(AF_LOCAL)
+  else if (!strncmp(authorization, "PeerCred ", 9) &&
+           con->http.hostaddr->addr.sa_family == AF_LOCAL)
+  {
+   /*
+    * Use peer credentials from domain socket connection...
+    */
+
+    struct passwd	*pwd;		/* Password entry for this user */
+    cupsd_ucred_t	peercred;	/* Peer credentials */
+    socklen_t		peersize;	/* Size of peer credentials */
+
+
+    if ((pwd = getpwnam(authorization + 9)) == NULL)
+    {
+      cupsdLogMessage(CUPSD_LOG_ERROR, "User \"%s\" does not exist!",
+                      authorization + 9);
+      return;
+    }
+
+    peersize = sizeof(peercred);
+
+    if (getsockopt(con->http.fd, SOL_SOCKET, SO_PEERCRED, &peercred, &peersize))
+    {
+      cupsdLogMessage(CUPSD_LOG_ERROR, "Unable to get peer credentials - %s",
+                      strerror(errno));
+      return;
+    }
+
+    if (pwd->pw_uid != CUPSD_UCRED_UID(peercred))
+    {
+      cupsdLogMessage(CUPSD_LOG_ERROR,
+                      "Invalid peer credentials for \"%s\" - got %d, "
+		      "expected %d!", authorization + 9,
+		      CUPSD_UCRED_UID(peercred), pwd->pw_uid);
+#  ifdef HAVE_SYS_UCRED_H
+      cupsdLogMessage(CUPSD_LOG_DEBUG, "cupsdAuthorize: cr_version=%d",
+                      peercred.cr_version);
+      cupsdLogMessage(CUPSD_LOG_DEBUG, "cupsdAuthorize: cr_uid=%d",
+                      peercred.cr_uid);
+      cupsdLogMessage(CUPSD_LOG_DEBUG, "cupsdAuthorize: cr_ngroups=%d",
+                      peercred.cr_ngroups);
+      cupsdLogMessage(CUPSD_LOG_DEBUG, "cupsdAuthorize: cr_groups[0]=%d",
+                      peercred.cr_groups[0]);
+#  endif /* HAVE_SYS_UCRED_H */
+      return;
+    }
+
+    strlcpy(username, authorization + 9, sizeof(username));
+
+    cupsdLogMessage(CUPSD_LOG_DEBUG,
+                    "cupsdAuthorize: Authorized as %s using PeerCred",
+		    username);
+
+    con->type = CUPSD_AUTH_BASIC;
+  }
+#endif /* SO_PEERCRED && AF_LOCAL */
   else if (!strncmp(authorization, "Local", 5) &&
            !strcasecmp(con->http.hostname, "localhost"))
   {
@@ -396,7 +541,13 @@ cupsdAuthorize(cupsd_client_t *con)	/* I - Client connection */
       authorization ++;
 
     if ((localuser = cupsdFindCert(authorization)) != NULL)
+    {
       strlcpy(username, localuser, sizeof(username));
+
+      cupsdLogMessage(CUPSD_LOG_DEBUG,
+		      "cupsdAuthorize: Authorized as %s using Local",
+		      username);
+    }
     else
     {
       cupsdLogMessage(CUPSD_LOG_ERROR,
@@ -404,9 +555,10 @@ cupsdAuthorize(cupsd_client_t *con)	/* I - Client connection */
 		      "found!");
       return;
     }
+
+    con->type = CUPSD_AUTH_BASIC;
   }
-  else if (!strncmp(authorization, "Basic", 5) &&
-           (type == AUTH_BASIC || type == AUTH_BASICDIGEST))
+  else if (!strncmp(authorization, "Basic", 5))
   {
    /*
     * Get the Basic authentication data...
@@ -465,7 +617,8 @@ cupsdAuthorize(cupsd_client_t *con)	/* I - Client connection */
 
     switch (type)
     {
-      case AUTH_BASIC :
+      default :
+      case CUPSD_AUTH_BASIC :
           {
 #if HAVE_LIBPAM
 	   /*
@@ -658,9 +811,13 @@ cupsdAuthorize(cupsd_client_t *con)	/* I - Client connection */
 	    }
 #endif /* HAVE_LIBPAM */
           }
+
+	  cupsdLogMessage(CUPSD_LOG_DEBUG,
+			  "cupsdAuthorize: Authorized as %s using Basic",
+			  username);
           break;
 
-      case AUTH_BASICDIGEST :
+      case CUPSD_AUTH_BASICDIGEST :
          /*
 	  * Do Basic authentication with the Digest password file...
 	  */
@@ -682,10 +839,16 @@ cupsdAuthorize(cupsd_client_t *con)	/* I - Client connection */
 	                    username);
             return;
 	  }
+
+	  cupsdLogMessage(CUPSD_LOG_DEBUG,
+			  "cupsdAuthorize: Authorized as %s using BasicDigest",
+			  username);
 	  break;
     }
+
+    con->type = type;
   }
-  else if (!strncmp(authorization, "Digest", 6) && type == AUTH_DIGEST)
+  else if (!strncmp(authorization, "Digest", 6))
   {
    /*
     * Get the username, password, and nonce from the Digest attributes...
@@ -753,11 +916,168 @@ cupsdAuthorize(cupsd_client_t *con)	/* I - Client connection */
 	              username);
       return;
     }
+
+    cupsdLogMessage(CUPSD_LOG_DEBUG,
+                    "cupsdAuthorize: Authorized as %s using Digest",
+		    username);
+
+    con->type = CUPSD_AUTH_DIGEST;
   }
+#ifdef HAVE_GSSAPI
+  else if (!strncmp(authorization, "Negotiate", 9)) 
+  {
+    int			len;		/* Length of authorization string */
+    gss_cred_id_t	server_creds;	/* Server credentials */
+    gss_ctx_id_t	context;	/* Authorization context */
+    OM_uint32		major_status,	/* Major status code */
+			minor_status;	/* Minor status code */
+    gss_buffer_desc	input_token = GSS_C_EMPTY_BUFFER,
+					/* Input token from string */
+			output_token = GSS_C_EMPTY_BUFFER;
+					/* Output token for username */
+    gss_name_t		client_name;	/* Client name */
+    unsigned int	ret_flags;	/* Credential flags */
+
+
+#  ifdef __APPLE__
+   /*
+    * If the weak-linked GSSAPI/Kerberos library is not present, don't try
+    * to use it...
+    */
+
+    if (gss_init_sec_context == NULL)
+    {
+      cupsdLogMessage(CUPSD_LOG_WARN,
+                      "GSSAPI/Kerberos authentication failed because the "
+		      "Kerberos framework is not present.");
+      return;
+    }
+#  endif /* __APPLE__ */
+
+    con->gss_output_token.length = 0;
+
+   /*
+    * Find the start of the Kerberos input token...
+    */
+
+    authorization += 9;
+    while (isspace(*authorization & 255))
+      authorization ++;
+
+    if (!*authorization)
+    {
+      cupsdLogMessage(CUPSD_LOG_DEBUG2,
+                      "cupsdAuthorize: No authentication data specified.");
+      return;
+    }
+
+   /*
+    * Get the server credentials...
+    */
+
+    if ((server_creds = get_gss_creds(GSSServiceName)) == NULL)
+      return;	
+
+   /*
+    * Decode the authorization string to get the input token...
+    */
+
+    len                = strlen(authorization);
+    input_token.value  = malloc(len);
+    input_token.value  = httpDecode64_2(input_token.value, &len,
+		                        authorization);
+    input_token.length = len;
+
+   /*
+    * Accept the input token to get the authorization info...
+    */
+
+    context      = GSS_C_NO_CONTEXT;
+    client_name  = GSS_C_NO_NAME;
+    major_status = gss_accept_sec_context(&minor_status,
+					  &context,
+					  server_creds, 
+					  &input_token,
+					  GSS_C_NO_CHANNEL_BINDINGS,
+					  &client_name,
+					  NULL,
+					  &con->gss_output_token,
+					  &ret_flags,
+					  NULL,
+					  &con->gss_delegated_cred);
+
+    if (GSS_ERROR(major_status))
+    {
+      cupsdLogGSSMessage(CUPSD_LOG_DEBUG, major_status, minor_status,
+                         "cupsdAuthorize: Error accepting GSSAPI security "
+			 "context");
+
+      if (context != GSS_C_NO_CONTEXT)
+	gss_delete_sec_context(&minor_status, &context, GSS_C_NO_BUFFER);
+
+      gss_release_cred(&minor_status, &server_creds);
+      return;
+    }
+
+   /*
+    * Release our credentials...
+    */
+
+    gss_release_cred(&minor_status, &server_creds);
+
+   /*
+    * Get the username associated with the client's credentials...
+    */
+
+    if (!con->gss_delegated_cred)
+      cupsdLogMessage(CUPSD_LOG_DEBUG,
+                      "cupsdAuthorize: No delegated credentials!");
+
+    if (major_status == GSS_S_CONTINUE_NEEDED)
+      cupsdLogGSSMessage(CUPSD_LOG_DEBUG, major_status, minor_status,
+                         "cupsdAuthorize: Credentials not complete");
+    else if (major_status == GSS_S_COMPLETE)
+    {
+      major_status = gss_display_name(&minor_status, client_name, 
+				      &output_token, NULL);
+
+      if (GSS_ERROR(major_status))
+      {
+	cupsdLogGSSMessage(CUPSD_LOG_DEBUG, major_status, minor_status,
+                           "cupsdAuthorize: Error getting username");
+	gss_release_name(&minor_status, &client_name);
+	gss_delete_sec_context(&minor_status, &context, GSS_C_NO_BUFFER);
+	return;
+      }
+
+      gss_release_name(&minor_status, &client_name);
+      strlcpy(username, output_token.value, sizeof(username));
+
+      cupsdLogMessage(CUPSD_LOG_DEBUG,
+		      "cupsdAuthorize: Authorized as %s using Negotiate",
+		      username);
+
+      gss_release_buffer(&minor_status, &output_token);
+      gss_delete_sec_context(&minor_status, &context, GSS_C_NO_BUFFER);
+
+      con->gss_have_creds = 1;
+
+      con->type = CUPSD_AUTH_NEGOTIATE;
+    }
+    else
+      gss_release_name(&minor_status, &client_name);
+  }
+#endif /* HAVE_GSSAPI */
   else
   {
-    cupsdLogMessage(CUPSD_LOG_DEBUG,
-                    "cupsdAuthorize: Bad authentication data.");
+    char	scheme[256];		/* Auth scheme... */
+
+
+    if (sscanf(authorization, "%255s", scheme) != 1)
+      strcpy(scheme, "UNKNOWN");
+
+    cupsdLogMessage(CUPSD_LOG_ERROR, "Bad authentication data \"%s ...\"",
+                    scheme);
     return;
   }
 
@@ -769,9 +1089,6 @@ cupsdAuthorize(cupsd_client_t *con)	/* I - Client connection */
 
   strlcpy(con->username, username, sizeof(con->username));
   strlcpy(con->password, password, sizeof(con->password));
-
-  cupsdLogMessage(CUPSD_LOG_DEBUG, "cupsdAuthorize: username=\"%s\"",
-                  con->username);
 }
 
 
@@ -798,7 +1115,7 @@ cupsdCheckAuth(
   {
     switch (masks->type)
     {
-      case AUTH_INTERFACE :
+      case CUPSD_AUTH_INTERFACE :
          /*
 	  * Check for a match with a network interface...
 	  */
@@ -906,7 +1223,7 @@ cupsdCheckAuth(
 	  }
 	  break;
 
-      case AUTH_NAME :
+      case CUPSD_AUTH_NAME :
          /*
 	  * Check for exact name match...
 	  */
@@ -925,7 +1242,7 @@ cupsdCheckAuth(
 	    return (1);
           break;
 
-      case AUTH_IP :
+      case CUPSD_AUTH_IP :
          /*
 	  * Check for IP/network address match...
 	  */
@@ -1116,7 +1433,7 @@ cupsdCopyLocation(
     for (i = 0; i < temp->num_allow; i ++)
       switch (temp->allow[i].type = (*loc)->allow[i].type)
       {
-        case AUTH_NAME :
+        case CUPSD_AUTH_NAME :
 	    temp->allow[i].mask.name.length = (*loc)->allow[i].mask.name.length;
 	    temp->allow[i].mask.name.name   = strdup((*loc)->allow[i].mask.name.name);
 
@@ -1129,7 +1446,7 @@ cupsdCopyLocation(
 	      return (NULL);
 	    }
 	    break;
-	case AUTH_IP :
+	case CUPSD_AUTH_IP :
 	    memcpy(&(temp->allow[i].mask.ip), &((*loc)->allow[i].mask.ip),
 	           sizeof(cupsd_ipmask_t));
 	    break;
@@ -1154,7 +1471,7 @@ cupsdCopyLocation(
     for (i = 0; i < temp->num_deny; i ++)
       switch (temp->deny[i].type = (*loc)->deny[i].type)
       {
-        case AUTH_NAME :
+        case CUPSD_AUTH_NAME :
 	    temp->deny[i].mask.name.length = (*loc)->deny[i].mask.name.length;
 	    temp->deny[i].mask.name.name   = strdup((*loc)->deny[i].mask.name.name);
 
@@ -1167,7 +1484,7 @@ cupsdCopyLocation(
 	      return (NULL);
 	    }
 	    break;
-	case AUTH_IP :
+	case CUPSD_AUTH_IP :
 	    memcpy(&(temp->deny[i].mask.ip), &((*loc)->deny[i].mask.ip),
 	           sizeof(cupsd_ipmask_t));
 	    break;
@@ -1227,14 +1544,14 @@ cupsdDeleteLocation(
     free(loc->names);
 
   for (i = loc->num_allow, mask = loc->allow; i > 0; i --, mask ++)
-    if (mask->type == AUTH_NAME || mask->type == AUTH_INTERFACE)
+    if (mask->type == CUPSD_AUTH_NAME || mask->type == CUPSD_AUTH_INTERFACE)
       free(mask->mask.name.name);
 
   if (loc->num_allow > 0)
     free(loc->allow);
 
   for (i = loc->num_deny, mask = loc->deny; i > 0; i --, mask ++)
-    if (mask->type == AUTH_NAME || mask->type == AUTH_INTERFACE)
+    if (mask->type == CUPSD_AUTH_NAME || mask->type == CUPSD_AUTH_INTERFACE)
       free(mask->mask.name.name);
 
   if (loc->num_deny > 0)
@@ -1271,7 +1588,7 @@ cupsdDenyHost(cupsd_location_t *loc,	/* I - Location to add to */
     * Deny *interface*...
     */
 
-    temp->type             = AUTH_INTERFACE;
+    temp->type             = CUPSD_AUTH_INTERFACE;
     temp->mask.name.name   = strdup("*");
     temp->mask.name.length = 1;
   }
@@ -1291,7 +1608,7 @@ cupsdDenyHost(cupsd_location_t *loc,	/* I - Location to add to */
       *ifptr = '\0';
     }
 
-    temp->type             = AUTH_INTERFACE;
+    temp->type             = CUPSD_AUTH_INTERFACE;
     temp->mask.name.name   = strdup(ifname);
     temp->mask.name.length = ifptr - ifname;
   }
@@ -1301,7 +1618,7 @@ cupsdDenyHost(cupsd_location_t *loc,	/* I - Location to add to */
     * Deny name...
     */
 
-    temp->type             = AUTH_NAME;
+    temp->type             = CUPSD_AUTH_NAME;
     temp->mask.name.name   = strdup(name);
     temp->mask.name.length = strlen(name);
   }
@@ -1315,8 +1632,8 @@ cupsdDenyHost(cupsd_location_t *loc,	/* I - Location to add to */
 
 void
 cupsdDenyIP(cupsd_location_t *loc,	/* I - Location to add to */
-	    unsigned         address[4],/* I - IP address to add */
-	    unsigned         netmask[4])/* I - Netmask of address */
+	    const unsigned   address[4],/* I - IP address to add */
+	    const unsigned   netmask[4])/* I - Netmask of address */
 {
   cupsd_authmask_t	*temp;		/* New host/domain mask */
 
@@ -1330,7 +1647,7 @@ cupsdDenyIP(cupsd_location_t *loc,	/* I - Location to add to */
   if ((temp = add_deny(loc)) == NULL)
     return;
 
-  temp->type = AUTH_IP;
+  temp->type = CUPSD_AUTH_IP;
   memcpy(temp->mask.ip.address, address, sizeof(temp->mask.ip.address));
   memcpy(temp->mask.ip.netmask, netmask, sizeof(temp->mask.ip.netmask));
 }
@@ -1351,22 +1668,22 @@ cupsdFindBest(const char   *path,	/* I - Resource path */
 			*best;		/* Best match for location so far */
   int			bestlen;	/* Length of best match */
   int			limit;		/* Limit field */
-  static const int	limits[] =	/* Map http_status_t to AUTH_LIMIT_xyz */
+  static const int	limits[] =	/* Map http_status_t to CUPSD_AUTH_LIMIT_xyz */
 		{
-		  AUTH_LIMIT_ALL,
-		  AUTH_LIMIT_OPTIONS,
-		  AUTH_LIMIT_GET,
-		  AUTH_LIMIT_GET,
-		  AUTH_LIMIT_HEAD,
-		  AUTH_LIMIT_POST,
-		  AUTH_LIMIT_POST,
-		  AUTH_LIMIT_POST,
-		  AUTH_LIMIT_PUT,
-		  AUTH_LIMIT_PUT,
-		  AUTH_LIMIT_DELETE,
-		  AUTH_LIMIT_TRACE,
-		  AUTH_LIMIT_ALL,
-		  AUTH_LIMIT_ALL
+		  CUPSD_AUTH_LIMIT_ALL,
+		  CUPSD_AUTH_LIMIT_OPTIONS,
+		  CUPSD_AUTH_LIMIT_GET,
+		  CUPSD_AUTH_LIMIT_GET,
+		  CUPSD_AUTH_LIMIT_HEAD,
+		  CUPSD_AUTH_LIMIT_POST,
+		  CUPSD_AUTH_LIMIT_POST,
+		  CUPSD_AUTH_LIMIT_POST,
+		  CUPSD_AUTH_LIMIT_PUT,
+		  CUPSD_AUTH_LIMIT_PUT,
+		  CUPSD_AUTH_LIMIT_DELETE,
+		  CUPSD_AUTH_LIMIT_TRACE,
+		  CUPSD_AUTH_LIMIT_ALL,
+		  CUPSD_AUTH_LIMIT_ALL
 		};
 
 
@@ -1476,11 +1793,14 @@ cupsdIsAuthorized(cupsd_client_t *con,	/* I - Connection */
                   const char     *owner)/* I - Owner of object */
 {
   int			i, j,		/* Looping vars */
-			auth;		/* Authorization status */
+			auth,		/* Authorization status */
+			type;		/* Type of authentication */
   unsigned		address[4];	/* Authorization address */
   cupsd_location_t	*best;		/* Best match for location so far */
   int			hostlen;	/* Length of hostname */
-  const char		*username;	/* Username to authorize */
+  char			username[256],	/* Username to authorize */
+			ownername[256],	/* Owner name to authorize */
+			*ptr;		/* Pointer into username */
   struct passwd		*pw;		/* User password data */
   static const char * const levels[] =	/* Auth levels */
 		{
@@ -1490,10 +1810,11 @@ cupsdIsAuthorized(cupsd_client_t *con,	/* I - Connection */
 		};
   static const char * const types[] =	/* Auth types */
 		{
-		  "NONE",
-		  "BASIC",
-		  "DIGEST",
-		  "BASICDIGEST"
+		  "None",
+		  "Basic",
+		  "Digest",
+		  "BasicDigest",
+		  "Negotiate"
 		};
 
 
@@ -1522,13 +1843,16 @@ cupsdIsAuthorized(cupsd_client_t *con,	/* I - Connection */
 
   best = con->best;
 
+  if ((type = best->type) == CUPSD_AUTH_DEFAULT)
+    type = DefaultAuthType;
+
   cupsdLogMessage(CUPSD_LOG_DEBUG2,
-                  "cupsdIsAuthorized: level=AUTH_%s, type=AUTH_%s, "
-		  "satisfy=AUTH_SATISFY_%s, num_names=%d",
-                  levels[best->level], types[best->type],
+                  "cupsdIsAuthorized: level=CUPSD_AUTH_%s, type=%s, "
+		  "satisfy=CUPSD_AUTH_SATISFY_%s, num_names=%d",
+                  levels[best->level], types[type],
 	          best->satisfy ? "ANY" : "ALL", best->num_names);
 
-  if (best->limit == AUTH_LIMIT_IPP)
+  if (best->limit == CUPSD_AUTH_LIMIT_IPP)
     cupsdLogMessage(CUPSD_LOG_DEBUG2, "cupsdIsAuthorized: op=%x(%s)",
                     best->op, ippOpString(best->op));
 
@@ -1572,7 +1896,7 @@ cupsdIsAuthorized(cupsd_client_t *con,	/* I - Connection */
     * Access from localhost (127.0.0.1 or ::1) is always allowed...
     */
 
-    auth = AUTH_ALLOW;
+    auth = CUPSD_AUTH_ALLOW;
   }
   else
   {
@@ -1583,39 +1907,39 @@ cupsdIsAuthorized(cupsd_client_t *con,	/* I - Connection */
     switch (best->order_type)
     {
       default :
-	  auth = AUTH_DENY;	/* anti-compiler-warning-code */
+	  auth = CUPSD_AUTH_DENY;	/* anti-compiler-warning-code */
 	  break;
 
-      case AUTH_ALLOW : /* Order Deny,Allow */
-          auth = AUTH_ALLOW;
+      case CUPSD_AUTH_ALLOW : /* Order Deny,Allow */
+          auth = CUPSD_AUTH_ALLOW;
 
           if (cupsdCheckAuth(address, con->http.hostname, hostlen,
 	          	best->num_deny, best->deny))
-	    auth = AUTH_DENY;
+	    auth = CUPSD_AUTH_DENY;
 
           if (cupsdCheckAuth(address, con->http.hostname, hostlen,
 	        	best->num_allow, best->allow))
-	    auth = AUTH_ALLOW;
+	    auth = CUPSD_AUTH_ALLOW;
 	  break;
 
-      case AUTH_DENY : /* Order Allow,Deny */
-          auth = AUTH_DENY;
+      case CUPSD_AUTH_DENY : /* Order Allow,Deny */
+          auth = CUPSD_AUTH_DENY;
 
           if (cupsdCheckAuth(address, con->http.hostname, hostlen,
 	        	best->num_allow, best->allow))
-	    auth = AUTH_ALLOW;
+	    auth = CUPSD_AUTH_ALLOW;
 
           if (cupsdCheckAuth(address, con->http.hostname, hostlen,
 	        	best->num_deny, best->deny))
-	    auth = AUTH_DENY;
+	    auth = CUPSD_AUTH_DENY;
 	  break;
     }
   }
 
-  cupsdLogMessage(CUPSD_LOG_DEBUG2, "cupsdIsAuthorized: auth=AUTH_%s...",
+  cupsdLogMessage(CUPSD_LOG_DEBUG2, "cupsdIsAuthorized: auth=CUPSD_AUTH_%s...",
                   auth ? "DENY" : "ALLOW");
 
-  if (auth == AUTH_DENY && best->satisfy == AUTH_SATISFY_ALL)
+  if (auth == CUPSD_AUTH_DENY && best->satisfy == CUPSD_AUTH_SATISFY_ALL)
     return (HTTP_FORBIDDEN);
 
 #ifdef HAVE_SSL
@@ -1623,11 +1947,13 @@ cupsdIsAuthorized(cupsd_client_t *con,	/* I - Connection */
   * See if encryption is required...
   */
 
-  if (best->encryption >= HTTP_ENCRYPT_REQUIRED && !con->http.tls &&
+  if ((best->encryption >= HTTP_ENCRYPT_REQUIRED && !con->http.tls &&
       strcasecmp(con->http.hostname, "localhost") &&
-      best->satisfy == AUTH_SATISFY_ALL)
+      best->satisfy == CUPSD_AUTH_SATISFY_ALL) &&
+      !(type == CUPSD_AUTH_NEGOTIATE || 
+        (type == CUPSD_AUTH_NONE && DefaultAuthType == CUPSD_AUTH_NEGOTIATE)))
   {
-    cupsdLogMessage(CUPSD_LOG_DEBUG2,
+    cupsdLogMessage(CUPSD_LOG_DEBUG,
                     "cupsdIsAuthorized: Need upgrade to TLS...");
     return (HTTP_UPGRADE_REQUIRED);
   }
@@ -1637,12 +1963,12 @@ cupsdIsAuthorized(cupsd_client_t *con,	/* I - Connection */
   * Now see what access level is required...
   */
 
-  if (best->level == AUTH_ANON ||	/* Anonymous access - allow it */
-      (best->type == AUTH_NONE && best->num_names == 0))
+  if (best->level == CUPSD_AUTH_ANON ||	/* Anonymous access - allow it */
+      (type == CUPSD_AUTH_NONE && best->num_names == 0))
     return (HTTP_OK);
 
-  if (!con->username[0] && best->type == AUTH_NONE &&
-      best->limit == AUTH_LIMIT_IPP)
+  if (!con->username[0] && type == CUPSD_AUTH_NONE &&
+      best->limit == CUPSD_AUTH_LIMIT_IPP)
   {
    /*
     * Check for unauthenticated username...
@@ -1654,30 +1980,43 @@ cupsdIsAuthorized(cupsd_client_t *con,	/* I - Connection */
     attr = ippFindAttribute(con->request, "requesting-user-name", IPP_TAG_NAME);
     if (attr)
     {
-      cupsdLogMessage(CUPSD_LOG_DEBUG2,
+      cupsdLogMessage(CUPSD_LOG_DEBUG,
                       "cupsdIsAuthorized: requesting-user-name=\"%s\"",
                       attr->values[0].string.text);
-      username = attr->values[0].string.text;
+      strlcpy(username, attr->values[0].string.text, sizeof(username));
     }
-    else if (best->satisfy == AUTH_SATISFY_ALL || auth == AUTH_DENY)
+    else if (best->satisfy == CUPSD_AUTH_SATISFY_ALL || auth == CUPSD_AUTH_DENY)
       return (HTTP_UNAUTHORIZED);	/* Non-anonymous needs user/pass */
     else
       return (HTTP_OK);			/* unless overridden with Satisfy */
   }
   else
   {
-    cupsdLogMessage(CUPSD_LOG_DEBUG2, "cupsdIsAuthorized: username=\"%s\"",
+    cupsdLogMessage(CUPSD_LOG_DEBUG, "cupsdIsAuthorized: username=\"%s\"",
 	            con->username);
 
+#ifdef HAVE_AUTHORIZATION_H
+    if (!con->username[0] && !con->authref)
+#else
     if (!con->username[0])
+#endif /* HAVE_AUTHORIZATION_H */
     {
-      if (best->satisfy == AUTH_SATISFY_ALL || auth == AUTH_DENY)
+      if (best->satisfy == CUPSD_AUTH_SATISFY_ALL || auth == CUPSD_AUTH_DENY)
 	return (HTTP_UNAUTHORIZED);	/* Non-anonymous needs user/pass */
       else
 	return (HTTP_OK);		/* unless overridden with Satisfy */
     }
 
-    username = con->username;
+    if (con->type != type && type != CUPSD_AUTH_NONE &&
+        (con->type != CUPSD_AUTH_BASIC || type != CUPSD_AUTH_BASICDIGEST))
+    {
+      cupsdLogMessage(CUPSD_LOG_ERROR, "Authorized using %s, expected %s!",
+                      types[con->type], types[type]);
+
+      return (HTTP_UNAUTHORIZED);
+    }
+
+    strlcpy(username, con->username, sizeof(username));
   }
 
  /*
@@ -1688,14 +2027,36 @@ cupsdIsAuthorized(cupsd_client_t *con,	/* I - Connection */
   if (!strcmp(username, "root"))
     return (HTTP_OK);
 
+ /*
+  * Strip any @domain or @KDC from the username and owner...
+  */
+
+  if ((ptr = strchr(username, '@')) != NULL)
+    *ptr = '\0';
+
+  if (owner)
+  {
+    strlcpy(ownername, owner, sizeof(ownername));
+
+    if ((ptr = strchr(ownername, '@')) != NULL)
+      *ptr = '\0';
+  }
+  else
+    ownername[0] = '\0';
+
  /*
   * Get the user info...
   */
 
-  pw = getpwnam(username);
-  endpwent();
+  if (username[0])
+  {
+    pw = getpwnam(username);
+    endpwent();
+  }
+  else
+    pw = NULL;
 
-  if (best->level == AUTH_USER)
+  if (best->level == CUPSD_AUTH_USER)
   {
    /*
     * If there are no names associated with this location, then
@@ -1713,10 +2074,32 @@ cupsdIsAuthorized(cupsd_client_t *con,	/* I - Connection */
     cupsdLogMessage(CUPSD_LOG_DEBUG2,
                     "cupsdIsAuthorized: Checking user membership...");
 
+#ifdef HAVE_AUTHORIZATION_H
+   /*
+    * If an authorization reference was supplied it must match a right name...
+    */
+
+    if (con->authref)
+    {
+      for (i = 0; i < best->num_names; i ++)
+      {
+	if (!strncasecmp(best->names[i], "@AUTHKEY(", 9) && 
+	    check_authref(con, best->names[i] + 9))
+	  return (HTTP_OK);
+	else if (!strcasecmp(best->names[i], "@SYSTEM") &&
+	         SystemGroupAuthKey &&
+		 check_authref(con, SystemGroupAuthKey))
+	  return (HTTP_OK);
+      }
+
+      return (HTTP_UNAUTHORIZED);
+    }
+#endif /* HAVE_AUTHORIZATION_H */
+
     for (i = 0; i < best->num_names; i ++)
     {
       if (!strcasecmp(best->names[i], "@OWNER") && owner &&
-          !strcasecmp(username, owner))
+          !strcasecmp(username, ownername))
 	return (HTTP_OK);
       else if (!strcasecmp(best->names[i], "@SYSTEM"))
       {
@@ -1767,7 +2150,7 @@ cupsdIsAuthorized(cupsd_client_t *con,	/* I - Connection */
   * The user isn't part of the specified group, so deny access...
   */
 
-  cupsdLogMessage(CUPSD_LOG_DEBUG2,
+  cupsdLogMessage(CUPSD_LOG_DEBUG,
                   "cupsdIsAuthorized: User not in group(s)!");
 
   return (HTTP_UNAUTHORIZED);
@@ -1858,6 +2241,59 @@ add_deny(cupsd_location_t *loc)		/* I - Location to add to */
 }
 
 
+#ifdef HAVE_AUTHORIZATION_H
+/*
+ * 'check_authref()' - Check if an authorization services reference has the
+ *		       supplied right.
+ */
+
+static int				/* O - 1 if right is valid, 0 otherwise */
+check_authref(cupsd_client_t *con,	/* I - Connection */
+	      const char     *right)	/* I - Right name */
+{
+  OSStatus		status;		/* OS Status */
+  AuthorizationItem	authright;	/* Authorization right */
+  AuthorizationRights	authrights;	/* Authorization rights */
+  AuthorizationFlags	authflags;	/* Authorization flags */
+
+
+ /*
+  * Check to see if the user is allowed to perform the task...
+  */
+
+  if (!con->authref)
+    return (0);
+
+  authright.name        = right;
+  authright.valueLength = 0;
+  authright.value       = NULL;
+  authright.flags       = 0;
+
+  authrights.count = 1;
+  authrights.items = &authright;
+
+  authflags = kAuthorizationFlagDefaults | 
+	      kAuthorizationFlagExtendRights;
+
+  if ((status = AuthorizationCopyRights(con->authref, &authrights, 
+					kAuthorizationEmptyEnvironment, 
+					authflags, NULL)) != 0)
+  {
+    cupsdLogMessage(CUPSD_LOG_ERROR,
+		    "AuthorizationCopyRights(\"%s\") returned %d (%s)",
+		    authright.name, (int)status, cssmErrorString(status));
+    return (0);
+  }
+
+  cupsdLogMessage(CUPSD_LOG_DEBUG2,
+                  "AuthorizationCopyRights(\"%s\") succeeded!",
+		  authright.name);
+
+  return (1);
+}
+#endif /* HAVE_AUTHORIZATION_H */
+
+
 /*
  * 'compare_locations()' - Compare two locations.
  */
@@ -1993,6 +2429,81 @@ cups_crypt(const char *pw,		/* I - Password string */
 #endif /* !HAVE_LIBPAM && !HAVE_USERSEC_H */
 
 
+#ifdef HAVE_GSSAPI
+/*
+ * 'get_gss_creds()' - Obtain GSS credentials.
+ */
+
+static gss_cred_id_t			/* O - Server credentials */
+get_gss_creds(const char *service_name)	/* I - Service name */
+{
+  OM_uint32	major_status,		/* Major status code */
+		minor_status;		/* Minor status code */
+  gss_name_t	server_name;		/* Server name */
+  gss_cred_id_t	server_creds;		/* Server credentials */
+  gss_buffer_desc token = GSS_C_EMPTY_BUFFER;
+					/* Service name token */
+  char		buf[1024],		/* Service name buffer */
+		fqdn[HTTP_MAX_URI];	/* Hostname of server */
+
+
+  snprintf(buf, sizeof(buf), "%s@%s", service_name,
+	   httpGetHostname(NULL, fqdn, sizeof(fqdn)));
+
+  token.value  = buf;
+  token.length = strlen(buf);
+  server_name  = GSS_C_NO_NAME;
+  major_status = gss_import_name(&minor_status, &token,
+	 			 GSS_C_NT_HOSTBASED_SERVICE,
+				 &server_name);
+
+  memset(&token, 0, sizeof(token));
+
+  if (GSS_ERROR(major_status))
+  {
+    cupsdLogGSSMessage(CUPSD_LOG_WARN, major_status, minor_status, 
+		       "gss_import_name() failed");
+    return (NULL);
+  }
+
+  major_status = gss_display_name(&minor_status, server_name, &token, NULL);
+
+  if (GSS_ERROR(major_status))
+  {
+    cupsdLogGSSMessage(CUPSD_LOG_WARN, major_status, minor_status,
+                       "gss_display_name() failed"); 
+    return (NULL);
+  }
+
+  cupsdLogMessage(CUPSD_LOG_DEBUG,
+                  "get_gss_creds: Attempting to acquire credentials for %s...", 
+                  (char *)token.value);
+
+  server_creds = GSS_C_NO_CREDENTIAL;
+  major_status = gss_acquire_cred(&minor_status, server_name, GSS_C_INDEFINITE,
+			          GSS_C_NO_OID_SET, GSS_C_ACCEPT,
+				  &server_creds, NULL, NULL);
+  if (GSS_ERROR(major_status))
+  {
+    cupsdLogGSSMessage(CUPSD_LOG_WARN, major_status, minor_status,
+                       "gss_acquire_cred() failed"); 
+    gss_release_name(&minor_status, &server_name);
+    gss_release_buffer(&minor_status, &token);
+    return (NULL);
+  }
+
+  cupsdLogMessage(CUPSD_LOG_DEBUG,
+                  "get_gss_creds: Credentials acquired successfully for %s.", 
+                  (char *)token.value);
+
+  gss_release_name(&minor_status, &server_name);
+  gss_release_buffer(&minor_status, &token);
+
+  return (server_creds);
+}
+#endif /* HAVE_GSSAPI */
+
+
 /*
  * 'get_md5_password()' - Get an MD5 password.
  */
@@ -2146,7 +2657,7 @@ pam_func(
 
   return (PAM_SUCCESS);
 }
-#else
+#elif !defined(HAVE_USERSEC_H)
 
 
 /*
@@ -2170,5 +2681,5 @@ to64(char          *s,			/* O - Output string */
 
 
 /*
- * End of "$Id: auth.c 5948 2006-09-12 13:58:39Z mike $".
+ * End of "$Id: auth.c 7319 2008-02-15 23:26:51Z mike $".
  */
