@@ -1,26 +1,17 @@
 /*
- * "$Id: cert.c 5381 2006-04-07 14:39:46Z mike $"
+ * "$Id: cert.c 7238 2008-01-22 01:41:31Z mike $"
  *
  *   Authentication certificate routines for the Common UNIX
  *   Printing System (CUPS).
  *
+ *   Copyright 2007-2008 by Apple Inc.
  *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  * Contents:
  *
@@ -51,7 +42,8 @@
 
 void
 cupsdAddCert(int        pid,		/* I - Process ID */
-             const char *username)	/* I - Username */
+             const char *username,	/* I - Username */
+             void       *ccache)	/* I - Kerberos credentials or NULL */
 {
   int		i;			/* Looping var */
   cupsd_cert_t	*cert;			/* Current certificate */
@@ -252,6 +244,16 @@ cupsdAddCert(int        pid,		/* I - Process ID */
   write(fd, cert->certificate, strlen(cert->certificate));
   close(fd);
 
+ /*
+  * Add Kerberos credentials as needed...
+  */
+
+#ifdef HAVE_GSSAPI
+  cert->ccache = (krb5_ccache)ccache;
+#else
+  (void)ccache;
+#endif /* HAVE_GSSAPI */
+
  /*
   * Insert the certificate at the front of the list...
   */
@@ -291,6 +293,15 @@ cupsdDeleteCert(int pid)		/* I - Process ID */
       else
         prev->next = cert->next;
 
+#ifdef HAVE_GSSAPI
+     /*
+      * Release Kerberos credentials as needed...
+      */
+
+      if (cert->ccache)
+	krb5_cc_destroy(KerberosContext, cert->ccache);
+#endif /* HAVE_GSSAPI */
+
       free(cert);
 
      /*
@@ -421,10 +432,10 @@ cupsdInitCerts(void)
   */
 
   if (!RunUser)
-    cupsdAddCert(0, "root");
+    cupsdAddCert(0, "root", NULL);
 }
 
 
 /*
- * End of "$Id: cert.c 5381 2006-04-07 14:39:46Z mike $".
+ * End of "$Id: cert.c 7238 2008-01-22 01:41:31Z mike $".
  */
