@@ -1,9 +1,9 @@
 /*
- * "$Id: main.c 7045 2007-10-31 05:42:13Z mike $"
+ * "$Id: main.c 7299 2008-02-12 01:00:19Z mike $"
  *
  *   Scheduler main loop for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 2007 by Apple Inc.
+ *   Copyright 2007-2008 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -133,10 +133,7 @@ main(int  argc,				/* I - Number of command-line args */
 			browse_time,	/* Next browse send time */
 			senddoc_time,	/* Send-Document time */
 			expire_time,	/* Subscription expire time */
-			mallinfo_time;	/* Malloc information time */
-  size_t		string_count,	/* String count */
-			alloc_bytes,	/* Allocated string bytes */
-			total_bytes;	/* Total string bytes */
+			report_time;	/* Malloc/client/job report time */
   long			timeout;	/* Timeout for cupsdDoSelect() */
   struct rlimit		limit;		/* Runtime limit */
 #if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
@@ -146,6 +143,12 @@ main(int  argc,				/* I - Number of command-line args */
   cups_file_t		*fp;		/* Fake lpsched lock file */
   struct stat		statbuf;	/* Needed for checking lpsched FIFO */
 #endif /* __sgi */
+#ifdef __APPLE__
+  int			run_as_child = 0;
+					/* Needed for Mac OS X fork/exec */
+#else
+  time_t		netif_time = 0;	/* Time since last network update */
+#endif /* __APPLE__ */
 #if HAVE_LAUNCHD
   int			launchd_idle_exit;
 					/* Idle exit on select timeout? */
@@ -183,6 +186,12 @@ main(int  argc,				/* I - Number of command-line args */
       for (opt = argv[i] + 1; *opt != '\0'; opt ++)
         switch (*opt)
 	{
+#ifdef __APPLE__
+	  case 'C' : /* Run as child with config file */
+              run_as_child = 1;
+	      fg           = -1;
+#endif /* __APPLE__ */
+
 	  case 'c' : /* Configuration file */
 	      i ++;
 	      if (i >= argc)
@@ -216,11 +225,22 @@ main(int  argc,				/* I - Number of command-line args */
 		* are passed a NULL pointer.
 	        */
 
-                current = malloc(1024);
-		getcwd(current, 1024);
+                if ((current = malloc(1024)) == NULL)
+		{
+		  _cupsLangPuts(stderr,
+		                _("cupsd: Unable to get current directory!\n"));
+                  return (1);
+		}
+
+		if (!getcwd(current, 1024))
+		{
+		  _cupsLangPuts(stderr,
+		                _("cupsd: Unable to get current directory!\n"));
+                  free(current);
+		  return (1);
+		}
 
 		cupsdSetStringf(&ConfigurationFile, "%s/%s", current, argv[i]);
-
 		free(current);
               }
 	      break;
@@ -334,6 +354,18 @@ main(int  argc,				/* I - Number of command-line args */
 	return (3);
       }
     }
+
+#ifdef __APPLE__
+   /*
+    * Since CoreFoundation has an overly-agressive check for whether a
+    * process has forked but not exec'd (whether CF has been called or
+    * not...), we now have to exec ourselves with the "-f" option to
+    * eliminate their bogus warning messages.
+    */
+
+    execlp(argv[0], argv[0], "-C", ConfigurationFile, (char *)0);
+    exit(errno);
+#endif /* __APPLE__ */
   }
 
   if (fg < 1)
@@ -574,7 +606,11 @@ main(int  argc,				/* I - Number of command-line args */
   * we are up and running...
   */
 
+#ifdef __APPLE__
+  if (!fg || run_as_child)
+#else
   if (!fg)
+#endif /* __APPLE__ */
   {
    /*
     * Send a signal to the parent process, but only if the parent is
@@ -606,11 +642,11 @@ main(int  argc,				/* I - Number of command-line args */
   * Loop forever...
   */
 
-  mallinfo_time = 0;
   browse_time   = time(NULL);
-  senddoc_time  = time(NULL);
   expire_time   = time(NULL);
   fds           = 1;
+  report_time   = 0;
+  senddoc_time  = time(NULL);
 
   while (!stop_scheduler)
   {
@@ -801,6 +837,18 @@ main(int  argc,				/* I - Number of command-line args */
 
     current_time = time(NULL);
 
+#ifndef __APPLE__
+   /*
+    * Update the network interfaces once a minute...
+    */
+
+    if ((current_time - netif_time) >= 60)
+    {
+      netif_time  = current_time;
+      NetIFUpdate = 1;
+    }
+#endif /* !__APPLE__ */
+
 #if HAVE_LAUNCHD
    /*
     * If no other work was scheduled and we're being controlled by launchd
@@ -877,7 +925,7 @@ main(int  argc,				/* I - Number of command-line args */
       */
 
       cupsdDeleteCert(0);
-      cupsdAddCert(0, "root");
+      cupsdAddCert(0, "root", NULL);
     }
 
    /*
@@ -925,30 +973,49 @@ main(int  argc,				/* I - Number of command-line args */
     }
 
    /*
-    * Log memory usage every minute...
+    * Log statistics at most once a minute when in debug mode...
     */
 
-    if ((current_time - mallinfo_time) >= 60 && LogLevel >= CUPSD_LOG_DEBUG2)
+    if ((current_time - report_time) >= 60 && LogLevel >= CUPSD_LOG_DEBUG)
     {
+      size_t		string_count,	/* String count */
+			alloc_bytes,	/* Allocated string bytes */
+			total_bytes;	/* Total string bytes */
 #ifdef HAVE_MALLINFO
-      struct mallinfo mem;		/* Malloc information */
+      struct mallinfo	mem;		/* Malloc information */
 
 
       mem = mallinfo();
-      cupsdLogMessage(CUPSD_LOG_DEBUG2,
-                      "mallinfo: arena = %d, used = %d, free = %d\n",
-                      mem.arena, mem.usmblks + mem.uordblks,
+      cupsdLogMessage(CUPSD_LOG_DEBUG, "Report: malloc-arena=%lu", mem.arena);
+      cupsdLogMessage(CUPSD_LOG_DEBUG, "Report: malloc-used=%lu",
+                      mem.usmblks + mem.uordblks);
+      cupsdLogMessage(CUPSD_LOG_DEBUG, "Report: malloc-free=%lu",
 		      mem.fsmblks + mem.fordblks);
 #endif /* HAVE_MALLINFO */
 
+      cupsdLogMessage(CUPSD_LOG_DEBUG, "Report: clients=%d",
+                      cupsArrayCount(Clients));
+      cupsdLogMessage(CUPSD_LOG_DEBUG, "Report: jobs=%d",
+                      cupsArrayCount(Jobs));
+      cupsdLogMessage(CUPSD_LOG_DEBUG, "Report: jobs-active=%d",
+                      cupsArrayCount(ActiveJobs));
+      cupsdLogMessage(CUPSD_LOG_DEBUG, "Report: printers=%d",
+                      cupsArrayCount(Printers));
+      cupsdLogMessage(CUPSD_LOG_DEBUG, "Report: printers-implicit=%d",
+                      cupsArrayCount(ImplicitPrinters));
+
       string_count = _cupsStrStatistics(&alloc_bytes, &total_bytes);
-      cupsdLogMessage(CUPSD_LOG_DEBUG2,
-                      "stringpool: " CUPS_LLFMT " strings, "
-		      CUPS_LLFMT " allocated, " CUPS_LLFMT " total bytes",
-		      CUPS_LLCAST string_count, CUPS_LLCAST alloc_bytes,
+      cupsdLogMessage(CUPSD_LOG_DEBUG,
+                      "Report: stringpool-string-count=" CUPS_LLFMT,
+		      CUPS_LLCAST string_count);
+      cupsdLogMessage(CUPSD_LOG_DEBUG,
+                      "Report: stringpool-alloc-bytes=" CUPS_LLFMT,
+		      CUPS_LLCAST alloc_bytes);
+      cupsdLogMessage(CUPSD_LOG_DEBUG,
+                      "Report: stringpool-total-bytes=" CUPS_LLFMT,
 		      CUPS_LLCAST total_bytes);
 
-      mallinfo_time = current_time;
+      report_time = current_time;
     }
 
    /*
@@ -959,7 +1026,9 @@ main(int  argc,				/* I - Number of command-line args */
     if (LastEvent)
     {
 #ifdef HAVE_NOTIFY_POST
-      if (LastEvent & CUPSD_EVENT_PRINTER_CHANGED)
+      if (LastEvent & (CUPSD_EVENT_PRINTER_ADDED |
+                       CUPSD_EVENT_PRINTER_DELETED |
+                       CUPSD_EVENT_PRINTER_MODIFIED))
       {
         cupsdLogMessage(CUPSD_LOG_DEBUG2,
 	                "notify_post(\"com.apple.printerListChange\")");
@@ -987,7 +1056,7 @@ main(int  argc,				/* I - Number of command-line args */
       * Reset the accumulated events...
       */
 
-      LastEvent     = CUPSD_EVENT_NONE;
+      LastEvent = CUPSD_EVENT_NONE;
     }
   }
 
@@ -1594,8 +1663,11 @@ process_children(void)
 	    else
  	      job->status = -status;	/* Backend failed */
 
-            if (job->printer && !(job->printer->type & CUPS_PRINTER_FAX))
+            if (job->printer && !(job->printer->type & CUPS_PRINTER_FAX) &&
+	        job->status_level > CUPSD_LOG_ERROR)
 	    {
+	      job->status_level = CUPSD_LOG_ERROR;
+
               snprintf(job->printer->state_message,
 	               sizeof(job->printer->state_message), "%s failed", name);
               cupsdAddPrinterHistory(job->printer);
@@ -1897,5 +1969,5 @@ usage(int status)			/* O - Exit status */
 
 
 /*
- * End of "$Id: main.c 7045 2007-10-31 05:42:13Z mike $".
+ * End of "$Id: main.c 7299 2008-02-12 01:00:19Z mike $".
  */
