@@ -1,26 +1,17 @@
 #!/bin/sh
 #
-# "$Id: 5.5-lp.sh,v 1.8 2005/01/03 19:30:00 mike Exp $"
+# "$Id: 5.5-lp.sh 6649 2007-07-11 21:46:42Z mike $"
 #
 #   Test the lp command.
 #
+#   Copyright 2007 by Apple Inc.
 #   Copyright 1997-2005 by Easy Software Products, all rights reserved.
 #
 #   These coded instructions, statements, and computer programs are the
-#   property of Easy Software Products and are protected by Federal
-#   copyright law.  Distribution and use rights are outlined in the file
-#   "LICENSE.txt" which should have been included with this file.  If this
-#   file is missing or damaged please contact Easy Software Products
-#   at:
-#
-#       Attn: CUPS Licensing Information
-#       Easy Software Products
-#       44141 Airport View Drive, Suite 204
-#       Hollywood, Maryland 20636 USA
-#
-#       Voice: (301) 373-9600
-#       EMail: cups-info@cups.org
-#         WWW: http://www.cups.org
+#   property of Apple Inc. and are protected by Federal copyright
+#   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+#   which should have been included with this file.  If this file is
+#   file is missing or damaged, see the license at "http://www.cups.org/".
 #
 
 echo "LP Default Test"
@@ -38,7 +29,7 @@ echo ""
 echo "LP Destination Test"
 echo ""
 echo "    lp -d Test1 testfile.jpg"
-../systemv/lp -d Test1 testfile.jpg 2>&1
+../systemv/lp -d Test1 -o job-hold-until=indefinite testfile.jpg 2>&1
 if test $? != 0; then
 	echo "    FAILED"
 	exit 1
@@ -67,5 +58,5 @@ fi
 echo ""
 
 #
-# End of "$Id: 5.5-lp.sh,v 1.8 2005/01/03 19:30:00 mike Exp $".
+# End of "$Id: 5.5-lp.sh 6649 2007-07-11 21:46:42Z mike $".
 #
